import java.awt.*;
import java.awt.image.*;
import java.lang.String.*;
import java.net.*;
import javax.swing.*;

/**
  * Up-Counter implementation of the <CODE>CircuitComponent</CODE> interface.
  *
  * @author Mike Cammarano
  * @author Charlie So
  */
public class UCTRComponent implements CircuitComponent 
{
  /**
    * The register used to hold this component's preset value.
    */
  private int presetLine;
  /**
    * The register used to hold this component's accumulator value.
    */
  private int accLine;
  /**
    * The initial preset value for this component.
    */
  private int presetVal;
  /**
    * The initial accumulator value for this component.
    */
  private int accVal;
  /**
    * The input to this component during the last call to the <CODE>step</CODE>
    * method.
    */
  private boolean last;
  /**
    * Whether this component should be drawn highlighted on the next repaint.
    */
  private boolean powered;
  /**
    * Image used to draw this component in a network when it is not powered.
    */
  private static Image pic;
  /**
    * Image used to draw this component in a network when it is powered.
    */
  private static Image picLit;
  /**
    * Cursor image used for this component during drag and drop.
    */
  private static Image cursorImage;
  /**
    * Icon used for this component in the component bin.
    */
  private static ImageIcon icon;
  /**
    * ToolTip text used for this component.
    */
  private static String tip = "Up Counter";
  /**
    * The simulations internal name for this component.
    */
  private static String name = "UCTR";

  static {
      // This static initializer loads all the image files needed
      URL myURL = ClassLoader.getSystemResource("BigUCTR.gif");
    	ImageIcon myIcon = new ImageIcon(myURL);
    	pic = myIcon.getImage();
      myURL = ClassLoader.getSystemResource("BigUCTROn.gif");
      myIcon = new ImageIcon(myURL);
      picLit = myIcon.getImage();
      myURL = ClassLoader.getSystemResource("CursorUCTR.gif");
      myIcon = new ImageIcon(myURL);
      cursorImage = myIcon.getImage();
      myURL = ClassLoader.getSystemResource("ToolUCTR.gif");
      icon = new ImageIcon(myURL);
  }

  /**
    * Construct an up-counter using default preset and accumulator values.
    */
  public UCTRComponent() {
    presetLine = 0;
    accLine = 1;
    presetVal = 0;
    accVal = 0;
  }

  /**
    * Returns 2, the number of ladder logic rungs occupied by this component.
    *
    * @return 2, the number of ladder logic rungs occupied by this component.
    */
  public int size() {
    return 2;
  }

  /**
    * Performs one evaluation of this component using the specified input
    * logic levels. If the 2nd rung of the counter is logic-low, then the
    * counter resets. Otherwise, the current input on the first rung is
    * compared to the last input to detect a positive transition. If the
    * counter sees a positive transition, and the accumulator hasn't yet
    * counted all the way up to the preset, then we increment the accumulator.
    *
    * @param b  array containing input logic levels of the rungs
    *           occupied by this component.
    *
    * @return the output logic levels from this component.
    */
  public boolean[] step(boolean[] b) {
    int tempAcc = SimMemory.GetReg(accLine);
    int tempPre = SimMemory.GetReg(presetLine);
    powered = false;
    if(b[1]) {
      if (!last && b[0] && tempAcc < tempPre) {
        powered = true;
        tempAcc++;
      }
    } else
      tempAcc = 0;

    last = b[0];
    SimMemory.SetReg(accLine, tempAcc);

    boolean tempArr[];
    tempArr = new boolean[2];

    tempArr[0] = (tempAcc == tempPre);
    tempArr[1] = (tempAcc <  tempPre);

    return tempArr;
  }

  /**
    * Displays a modal dialog box to allow the user to edit the parameters
    * of this component, such as which registers are used. This method will
    * initialize the state of the existing instance of <CODE>CounterDialog</CODE>
    * in <CODE>Master</CODE>, show it, and then update the accumulator and
    * preset of this counter based on the user's selections in that dialog.
    */
  public void edit() {
    SimMemory.deleteUser(presetLine, this);
    SimMemory.deleteUser(accLine, this);
    Master.DialogCounter.setState(presetLine, accLine, presetVal, accVal, this);
    Master.DialogCounter.setVisible(true);
    accLine = Master.DialogCounter.getAccLine();
    presetLine = Master.DialogCounter.getPresetLine();
    accVal = Master.DialogCounter.getAccVal();
    presetVal = Master.DialogCounter.getPresetVal();
    SimMemory.addUser(presetLine, this);
    SimMemory.addUser(accLine, this);
  }

  /**
    * Clears any registers marked as used by this component prior to
    * its deletion.
    */
  public void clear() {
    SimMemory.deleteUser(presetLine, this);
    SimMemory.deleteUser(accLine, this);
  }

  /**
    * Resets this component, and initializes the preset and accumulator values
    * in <CODE>SimMemory</CODE>.
    */
  public void reset() {
    SimMemory.SetReg(presetLine, presetVal);
    SimMemory.SetReg(accLine, accVal);
    last = false;
    powered = false;
  }

  /**
    * Returns the text to be used as a tool tip when this component
    * is placed in the component bin.
    *
    * @return the string to display.
    */
  public String getTip() {
    return tip;
  }

  /**
    * Returns the name of this component to be used internally in saved files
    * and in drag and drop operations.
    *
    * @return the string to use.
    */
  public String getName() {
    return name;
  }

  /**
    * Returns the Icon to be used for this component in the component bin.
    *
    * @return the icon.
    */
  public ImageIcon getToolIcon() {
    return icon;
  }

  /**
    * Returns a one line string representing the state of this component for use in
    * a save file. The string will be of the form:
    * <BR>
    * UCTR <I>preset-register accumulator-register preset-value accumulator-value</I>.
    *
    * @return the parameter string.
    */
  public String getSaveData() {
    return name + " " + presetLine + " " + accLine + " " + presetVal + " " + accVal;
  }

  /**
    * Returns whether this component must be the rightmost component on any rung.
    * For example, an output coil terminates a rung; a switch does not.
    *
    * @return <CODE>False</CODE> since counters don't terminate rungs.
    */
  public boolean terminatesRung() {
    return false;
  }

  /**
    * Returns the image to use as a mouse cursor for this component during
    * a drag and drop operation.
    *
    * @return the cursor image.
    */
  public Image getImage() {
    return cursorImage;
  }

  /**
    * Returns a new instance of this component created using the default constructor.
    * This method is used by the <CODE>getComponent</CODE> factory method of
    * <CODE>Elements</CODE>.
    *
    * @return the new <CODE>CircuitComponent</CODE> instance.
    */
  public CircuitComponent getInstance() {
    UCTRComponent x = new UCTRComponent();
    SimMemory.addUser(x.presetLine, x);
    SimMemory.addUser(x.accLine, x);
    return x;
  }

  /**
    * Returns a new instance of this component, using the specified initial parameters.
    * This method is used by the <CODE>getComponent</CODE> factory method of
    * <CODE>Elements</CODE> to instantiate new components during a file open.
    *
    * @return the new <CODE>CircuitComponent</CODE> instance.
    */
  public CircuitComponent getInstance(int[] a) {
    UCTRComponent x = new UCTRComponent();
    x.presetLine = a[0];
    x.accLine = a[1];
    x.presetVal = a[2];
    x.accVal = a[3];
    SimMemory.addUser(x.presetLine, x);
    SimMemory.addUser(x.accLine, x);
    return x;
  }

  /**
    * Draws this component and any associated parameters at the specified coordinates
    * in the graphics context. The preset value is written in the top half of the
    * counter, and the initial accumulator value is written in the bottom half.
    *
    * @param g  the graphics context in which to draw.
    * @param x  the x-coordinate of the northwest corner.
    * @param y  the y-coordinate of the northwest corner.
    * @param o  the image observer to be notified of image drawing.
    */
  public void drawComponent(Graphics g, int x, int y, ImageObserver o) {
    if(Master.isSimActive && powered) {
      // Draw a powered component
      g.setColor(Network.darkGreen);
      g.drawImage(picLit, x, y, o);
    } else {
      // Draw a powered component
      g.drawImage(pic, x, y, o);
    }

    // Draw the preset value in the top half of the component
    String text;
    text = "     " + Integer.toString(presetVal);
    text = text.substring(text.length()-5);
    g.drawString(text, x+10, y+16);

    // Draw the accumulator value in the bottom half of the component
    text = "     " + Integer.toString(accVal);
    text = text.substring(text.length()-5);
    g.drawString(text, x+10, y+40);
      g.setColor(Color.black);
  }
}
